//-----------------------------------------------------------------------
// File : AsuraModelImpl.cpp
// Desc : Asura Model Implemenation
// Copyright(c) Project Asura All right reserved.
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
// History
//-----------------------------------------------------------------------
// <---> 2010/11/13 [Pocol] VK쐬.
// <End>

//-----------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------
#include "asModelImpl.h"
#include "asModelLoaderX.h"


//-----------------------------------------------------------------------
// Using Statements
//-----------------------------------------------------------------------
using namespace Asura::Math;


namespace Asura {
namespace Graphics {


/////////////////////////////////////////////////////////////////////////
// UShort4 structure
/////////////////////////////////////////////////////////////////////////
UShort4::UShort4()
{
    memset( val, 0, sizeof(uint16_t) * 4 );
};

UShort4::UShort4(const uint16_t *pValues)
{
    assert( pValues != NULL );
    memcpy( val, pValues, sizeof(uint16_t) * 4 );
}

UShort4::UShort4( uint16_t val0, uint16_t val1, uint16_t val2, uint16_t val3 )
{
    val[0] = val0;
    val[1] = val1;
    val[2] = val2;
    val[3] = val3;
}

UShort4::operator uint16_t* ()
{
    return (uint16_t *)&val[0];
}

UShort4::operator const uint16_t *() const
{
    return (const uint16_t *)&val[0];
}

bool UShort4::operator == ( const UShort4 &value )
{
    return ( ( val[0] == value.val[0] ) 
          && ( val[1] == value.val[1] )
          && ( val[2] == value.val[2] )
          && ( val[3] == value.val[3] ) );
}

bool UShort4::operator != ( const UShort4 &value )
{
    return ( ( val[0] != value.val[0] )
          || ( val[1] != value.val[1] )
          || ( val[2] != value.val[2] )
          || ( val[3] != value.val[3] ) );
}

/////////////////////////////////////////////////////////////////////////
// Short4 structure
/////////////////////////////////////////////////////////////////////////
Short4::Short4()
{
    memset( val, 0, sizeof(int16_t) * 4 );
};

Short4::Short4(const int16_t *pValues)
{
    assert( pValues != NULL );
    memcpy( val, pValues, sizeof(int16_t) * 4 );
}

Short4::Short4( int16_t val0, int16_t val1, int16_t val2, int16_t val3 )
{
    val[0] = val0;
    val[1] = val1;
    val[2] = val2;
    val[3] = val3;
}

Short4::operator int16_t* ()
{
    return (int16_t *)&val[0];
}

Short4::operator const int16_t *() const
{
    return (const int16_t *)&val[0];
}

bool Short4::operator == ( const Short4 &value )
{
    return ( ( val[0] == value.val[0] ) 
          && ( val[1] == value.val[1] )
          && ( val[2] == value.val[2] )
          && ( val[3] == value.val[3] ) );
}

bool Short4::operator != ( const Short4 &value )
{
    return ( ( val[0] != value.val[0] )
          || ( val[1] != value.val[1] )
          || ( val[2] != value.val[2] )
          || ( val[3] != value.val[3] ) );
}

/////////////////////////////////////////////////////////////////////////
// UByte4 structure
/////////////////////////////////////////////////////////////////////////
UByte4::UByte4()
{
    memset( val, 0, sizeof(uint8_t) * 4 );
};

UByte4::UByte4(const uint8_t *pValues)
{
    assert( pValues != NULL );
    memcpy( val, pValues, sizeof(uint8_t) * 4 );
}

UByte4::UByte4( uint8_t val0, uint8_t val1, uint8_t val2, uint8_t val3 )
{
    val[0] = val0;
    val[1] = val1;
    val[2] = val2;
    val[3] = val3;
}

UByte4::operator uint8_t* ()
{
    return (uint8_t *)&val[0];
}

UByte4::operator const uint8_t *() const
{
    return (const uint8_t *)&val[0];
}

bool UByte4::operator == ( const UByte4 &value )
{
    return ( ( val[0] == value.val[0] ) 
          && ( val[1] == value.val[1] )
          && ( val[2] == value.val[2] )
          && ( val[3] == value.val[3] ) );
}

bool UByte4::operator != ( const UByte4 &value )
{
    return ( ( val[0] != value.val[0] )
          || ( val[1] != value.val[1] )
          || ( val[2] != value.val[2] )
          || ( val[3] != value.val[3] ) );
}

/////////////////////////////////////////////////////////////////////////
// Byte4 structure
/////////////////////////////////////////////////////////////////////////
Byte4::Byte4()
{
    memset( val, 0, sizeof(int8_t) * 4 );
};

Byte4::Byte4(const int8_t *pValues)
{
    assert( pValues != NULL );
    memcpy( val, pValues, sizeof(int8_t) * 4 );
}

Byte4::Byte4( int8_t val0, int8_t val1, int8_t val2, int8_t val3 )
{
    val[0] = val0;
    val[1] = val1;
    val[2] = val2;
    val[3] = val3;
}

Byte4::operator int8_t* ()
{
    return (int8_t *)&val[0];
}

Byte4::operator const int8_t *() const
{
    return (const int8_t *)&val[0];
}

bool Byte4::operator == ( const Byte4 &value )
{
    return ( ( val[0] == value.val[0] ) 
          && ( val[1] == value.val[1] )
          && ( val[2] == value.val[2] )
          && ( val[3] == value.val[3] ) );
}

bool Byte4::operator != ( const Byte4 &value )
{
    return ( ( val[0] != value.val[0] )
          || ( val[1] != value.val[1] )
          || ( val[2] != value.val[2] )
          || ( val[3] != value.val[3] ) );
}


////////////////////////////////////////////////////////////////////////
// Bone class
////////////////////////////////////////////////////////////////////////

///---------------------------------------------------------------------
///<summary>
///RXgN^
///</summary>
///---------------------------------------------------------------------
Bone::Bone()
{
    mName.clear();
    mParentIndex = -1;
    mBindMatrix.Identity();
    mPoseMatrix.Identity();
}

///---------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///---------------------------------------------------------------------
Bone::~Bone()
{
    /* DO NOTHING */
}

///---------------------------------------------------------------------
///<summary>
///
///</summary>
///---------------------------------------------------------------------
void Bone::Release()
{
    mName.clear();
}

///---------------------------------------------------------------------
///<summary>
///O擾
///</summary>
///<returns>{[ԋp</returns>
///---------------------------------------------------------------------
std::string Bone::GetName()
{ return mName; }

///---------------------------------------------------------------------
///<summary>
///ẽ{[CfbNX擾
///</summary>
///<returns>
///ẽ{[CfbNXԋp.
///eȂ[g̏ꍇ-1ԋp.
///</returns>
///---------------------------------------------------------------------
int32_t Bone::GetParentIndex()
{ return mParentIndex; }

///---------------------------------------------------------------------
///<summary>
///{[Wn֕ϊs擾
///</summary>
///<returns>{[Wnւƕϊsԋp</returns>
///---------------------------------------------------------------------
Matrix Bone::GetBindMatrix()
{ return mBindMatrix; }

///---------------------------------------------------------------------
///<summary>
///ps擾
///</summary>
///<returns>p肷ItZbgsԋp</returns>
///---------------------------------------------------------------------
Matrix Bone::GetPoseMatrix()
{ return mPoseMatrix; }

///---------------------------------------------------------------------
///<summary>
///Oݒ肷
///</summary>
///<param name="name">ݒ肷閼O</param>
///---------------------------------------------------------------------
void Bone::SetName( const std::string &name )
{ mName = name; }

///---------------------------------------------------------------------
///<summary>
///ẽ{[CfbNXݒ肷
///</summary>
///<param name="index">ݒ肷ẽ{[CfbNX</param>
///---------------------------------------------------------------------
void Bone::SetParentIndex( const int32_t &index )
{ mParentIndex = index; }

///---------------------------------------------------------------------
///<summary>
///{[Wnւƕϊsݒ肷
///</summary>
///<param name="matrix">ݒ肷{[Wnւƕϊs</param>
///---------------------------------------------------------------------
void Bone::SetBindMatrix( const Matrix &matrix )
{ mBindMatrix = matrix; }

///---------------------------------------------------------------------
///<summary>
///psݒ肷
///</summary>
///<param name="matrix">ݒ肷ps</param>
///---------------------------------------------------------------------
void Bone::SetPoseMatrix( const Matrix &matrix )
{ mPoseMatrix = matrix; }

Bone& Bone::operator =( const Bone &bone )
{
    mName = bone.mName;
    mParentIndex = bone.mParentIndex;
    mBindMatrix = bone.mBindMatrix;
    mPoseMatrix = bone.mPoseMatrix;
    return (*this);
}

////////////////////////////////////////////////////////////////////////
// Material class
////////////////////////////////////////////////////////////////////////

///---------------------------------------------------------------------
///<summary>
///RXgN^
///</summary>
///---------------------------------------------------------------------
Material::Material()
{
    mDiffuse = Vector4( 0.0f, 0.0f, 0.0f, 0.0f );
    mSpecular = Vector3( 0.0f, 0.0f, 0.0f );
    mEmissive = Vector3( 0.0f, 0.0f, 0.0f );
    mPower = 0.0f;
    mTextureName.clear();
    mName.clear();
}

///---------------------------------------------------------------------
///<summary>
///Rs[RXgN^
///</summary>
///<param name="material">Rs[̃CX^X</param>
///---------------------------------------------------------------------
Material::Material( const Material &material )
{
    mDiffuse = material.mDiffuse;
    mSpecular = material.mSpecular;
    mEmissive = material.mEmissive;
    mPower = material.mPower;
	mTextureName = material.mTextureName;
    mName = material.mName;
}

///---------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///---------------------------------------------------------------------
Material::~Material()
{
	Release();
}

///---------------------------------------------------------------------
///<summary>
///
///</summary>
///---------------------------------------------------------------------
void Material::Release()
{
    mTextureName.clear();
    mName.clear();
}

///---------------------------------------------------------------------
///<summary>
///gUˍގ擾
///</summary>
///<returns>gUˍގԋp</returns>
///---------------------------------------------------------------------
Vector4 Material::GetDiffuse()
{ return mDiffuse; }

///---------------------------------------------------------------------
///<summary>
///ʔˍގ擾
///</summary>
///<returns>ʔˍގԋp</returns>
///---------------------------------------------------------------------
Vector3 Material::GetSpecular()
{ return mSpecular; }

///---------------------------------------------------------------------
///<summary>
///ˍގ擾
///</summary>
///<returns>ˍގԋp</returns>
///---------------------------------------------------------------------
Vector3 Material::GetEmissive()
{ return mEmissive; }

///---------------------------------------------------------------------
///<summary>
///ʔˎw擾
///</summary>
///<returns>ʔˎwԋp</returns>
///---------------------------------------------------------------------
float Material::GetPower()
{ return mPower; }

///---------------------------------------------------------------------
///<summary>
///eNX`t@C擾
///</summary>
///<returns>eNX`t@Cԋp</returns>
///---------------------------------------------------------------------
std::string Material::GetTextureName()
{
    return mTextureName;
}

///---------------------------------------------------------------------
///<summary>
///}eA擾
///</summary>
///<returns>}eAԋp</returns>
///---------------------------------------------------------------------
std::string Material::GetName()
{ return mName; }

///---------------------------------------------------------------------
///<summary>
///}eAݒ肷
///</summary>
///<param name="name">ݒ肷}eA</param>
///---------------------------------------------------------------------
void Material::SetName( const std::string &name )
{ mName = name; }

///---------------------------------------------------------------------
///<summary>
///gUˍގݒ肷
///</summary>
///<param name="diffuse">ݒ肷gUˍގ</param>
///---------------------------------------------------------------------
void Material::SetDiffuse( const Vector4 &diffuse )
{ mDiffuse = diffuse; }

///---------------------------------------------------------------------
///<summary>
///ʔˍގݒ肷
///</summary>
///<param name="specular">ݒ肷鋾ʔˍގ</param>
///---------------------------------------------------------------------
void Material::SetSpecular( const Vector3 &specular )
{ mSpecular = specular; }

///---------------------------------------------------------------------
///<summary>
///ˍގݒ肷
///</summary>
///<param name="emissive">ݒ肷ˍގ</param>
///---------------------------------------------------------------------
void Material::SetEmissive( const Vector3 &emissive )
{ mEmissive = emissive; }

///---------------------------------------------------------------------
///<summary>
///ʔˎwݒ肷
///</summary>
///<param name="power">ݒ肷鋾ʔˎw</param>
///---------------------------------------------------------------------
void Material::SetPower( const float &power )
{ mPower = power; }

///---------------------------------------------------------------------
///<summary>
///eNX`t@Cݒ肷
///</summary>
///<param name="texture_name">ݒ肷eNX`t@C</param>
///---------------------------------------------------------------------
void Material::SetTextureName( const std::string &texture_name )
{ mTextureName = texture_name; }

Material& Material::operator =( const Material &material )
{
    mDiffuse = material.mDiffuse;
    mSpecular = material.mSpecular;
    mEmissive = material.mEmissive;
    mPower = material.mPower;
	mTextureName = material.mTextureName;
    mName = material.mName;
    return (*this);
}

////////////////////////////////////////////////////////////////////////
// MeshSubset class
////////////////////////////////////////////////////////////////////////

///---------------------------------------------------------------------
///<summary>
///RXgN^
///</summary>
///---------------------------------------------------------------------
MeshSubset::MeshSubset()
{
    mMaterialIndex = 0;
    mFaceIndex = 0;
    mNumFace = 0;
}

///---------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///---------------------------------------------------------------------
MeshSubset::~MeshSubset()
{
    /* DO NOTHING */
}

///---------------------------------------------------------------------
///<summary>
///}eACfbNX擾
///</summary>
///<returns>}eACfbNXԋp</returns>
///---------------------------------------------------------------------
uint32_t MeshSubset::GetMaterialIndex()
{ return mMaterialIndex; }

///---------------------------------------------------------------------
///<summary>
///ʂ̃CfbNX擾
///</summary>
///<returns>Jnʂ̃CfbNX̒lԋp</returns>
///---------------------------------------------------------------------
uint32_t MeshSubset::GetFaceIndex()
{ return mFaceIndex; }

///---------------------------------------------------------------------
///<summary>
///ʐ擾
///</summary>
///<returns>`悷ʐԋp</returns>
///---------------------------------------------------------------------
uint32_t MeshSubset::GetNumFace()
{ return mNumFace; }

///---------------------------------------------------------------------
///<summary>
///}eACfbNXݒ肷
///</summary>
///<param name="material_index">ݒ肷}eACfbNX</param>
///---------------------------------------------------------------------
void MeshSubset::SetMaterialIndex( const uint32_t &material_index )
{ mMaterialIndex = material_index; }

///---------------------------------------------------------------------
///<summary>
///ʂ̃CfbNXݒ肷
///</summary>
///<param name="face_index">ݒ肷ʂ̃CfbNX</param>
///---------------------------------------------------------------------
void MeshSubset::SetFaceIndex( const uint32_t &face_index )
{ mFaceIndex = face_index; }

///---------------------------------------------------------------------
///<summary>
///ʐݒ肷
///</summary>
///<param name="num_face">ݒ肷`ʐ</param>
///---------------------------------------------------------------------
void MeshSubset::SetNumFace( const uint32_t &num_face )
{ mNumFace = num_face; }

MeshSubset& MeshSubset::operator = ( const MeshSubset &subset )
{
    mMaterialIndex = subset.mMaterialIndex;
    mFaceIndex = subset.mFaceIndex;
    mNumFace = subset.mNumFace;
    return (*this);
}

////////////////////////////////////////////////////////////////////////
// Mesh class
////////////////////////////////////////////////////////////////////////

///---------------------------------------------------------------------
///<summary>
///RXgN^
///</summary>
///---------------------------------------------------------------------
Mesh::Mesh()
{
	mPositions.clear();
	mNormals.clear();
	mColors.clear();
	mTexCoords.clear();
    mTangents.clear();
    mBinormals.clear();
	mSkinIndices.clear();
	mSkinWeights.clear();
	mIndices.clear();
    mBones.clear();
	mSubsets.clear();
	mMaterials.clear();
}

///---------------------------------------------------------------------
///<summary>
///Rs[RXgN^
///</summary>
///<param name="mesh">Rs[̃CX^X</param>
///---------------------------------------------------------------------
Mesh::Mesh( const Mesh &mesh )
{
	SetName( mesh.mName );
	SetPositions( mesh.mPositions );
	SetNormals( mesh.mNormals );
	SetColors( mesh.mColors );
	SetTexCoords( mesh.mTexCoords );
	SetTangents( mesh.mTangents );
	SetBinormals( mesh.mBinormals );
	SetSkinIndices( mesh.mSkinIndices );
	SetSkinWeights( mesh.mSkinWeights );
    SetIndices( mesh.mIndices );
    SetBones( mesh.mBones );
	SetSubsets( mesh.mSubsets );
	SetMaterials( mesh.mMaterials );
}

///---------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///---------------------------------------------------------------------
Mesh::~Mesh()
{
    /* DO_NOTHING */
}

///---------------------------------------------------------------------
///<summary>
///
///</summary>
///---------------------------------------------------------------------
void Mesh::Release()
{
	for( size_t i=0; i<mBones.size(); i++ )
	{
		mBones[i].Release();
	}
	for( size_t i=0; i<mMaterials.size(); i++ )
	{
		mMaterials[i].Release();
	}
	mPositions.clear();
	mNormals.clear();
	mColors.clear();
	mTexCoords.clear();
    mTangents.clear();
    mBinormals.clear();
	mSkinIndices.clear();
	mSkinWeights.clear();
	mIndices.clear();
    mBones.clear();
	mSubsets.clear();
	mMaterials.clear();
}

///---------------------------------------------------------------------
///<summary>
///bV擾
///</summary>
///<returns>bVԋp</returns>
///---------------------------------------------------------------------
std::string Mesh::GetName()
{ return mName; }

///---------------------------------------------------------------------
///<summary>
///ʒuWRei擾
///</summary>
///<returns>ʒuWReiԋp</returns>
///---------------------------------------------------------------------
std::vector< Vector3 > Mesh::GetPositions()
{
    return mPositions;
}

///---------------------------------------------------------------------
///<summary>
///@xNgRei擾
///</summary>
///<returns>@xNgReiԋp</returns>
///---------------------------------------------------------------------
std::vector< Vector3 > Mesh::GetNormals()
{
    return mNormals;
}

///---------------------------------------------------------------------
///<summary>
///_J[Rei擾
///</summary>
///<returns>_J[Reiԋp</returns>
///---------------------------------------------------------------------
std::vector< UByte4 > Mesh::GetColors()
{
    return mColors;
}

///---------------------------------------------------------------------
///<summary>
///eNX`WRei擾
///</summary>
///<returns>eNX`WReiԋp</returns>
///---------------------------------------------------------------------
std::vector< Vector2 > Mesh::GetTexCoords()
{
    return mTexCoords;
}

///---------------------------------------------------------------------
///<summary>
///ڃxNgRei擾
///</summary>
///<returns>ڃxNgReiԋp</returns>
///---------------------------------------------------------------------
std::vector< Vector3 > Mesh::GetTangents()
{
    return mTangents;
}

///---------------------------------------------------------------------
///<summary>
///]@xNgRei擾
///</summary>
///<returns>]@xNgReiԋp</returns>
///---------------------------------------------------------------------
std::vector< Vector3 > Mesh::GetBinormals()
{
    return mBinormals;
}

///---------------------------------------------------------------------
///<summary>
///XLCfbNXRei擾
///</summary>
///<returns>XLCfbNXReiԋp</returns>
///---------------------------------------------------------------------
std::vector< Short4 > Mesh::GetSkinIndices()
{
    return mSkinIndices;
}

///---------------------------------------------------------------------
///<summary>
///XLEFCgRei擾
///</summary>
///<returns>XLEFCgReiԋp</returns>
///---------------------------------------------------------------------
std::vector< Vector4 > Mesh::GetSkinWeights()
{
    return mSkinWeights;
}

///---------------------------------------------------------------------
///<summary>
///_CfbNXRei擾
///</summary>
///<returns>_CfbNXReiԋp</returns>
///---------------------------------------------------------------------
std::vector< uint32_t > Mesh::GetIndices()
{
    return mIndices;
}

///---------------------------------------------------------------------
///<summary>
///{[Rei擾
///</summary>
///<returns>IBone*^ɕϊ{[Reiԋp</returns>
///---------------------------------------------------------------------
std::vector< IBone* > Mesh::GetBones()
{
    std::vector< IBone* > result;
    result.resize( mBones.size() );
    for( size_t i=0; i<mBones.size(); i++ )
    {
        result[i] = &mBones[i];
    }
    return result;
}

///--------------------------------------------------------------------
///<summary>
///TuZbgRei擾
///</summary>
///<returns>IMeshSubset*^ɕϊTuZbgReiԋp</returns>
///--------------------------------------------------------------------
std::vector< IMeshSubset* > Mesh::GetSubsets()
{
    std::vector< IMeshSubset* > result;
    result.resize( mSubsets.size() );
    for( size_t i=0; i<mSubsets.size(); i++ )
    {
        result[i] = &mSubsets[i];
    }
    return result;
}

///---------------------------------------------------------------------
///<summary>
///}eAReiԋp
///</summary>
///<returns>IMaterial*^ɕϊ}eAReiԋp</returns>
///---------------------------------------------------------------------
std::vector< IMaterial* > Mesh::GetMaterials()
{
    std::vector< IMaterial* > result;
    result.resize( mMaterials.size() );
    for( size_t i=0; i<mMaterials.size(); i++ )
    {
        result[i] = &mMaterials[i];
    }
    return result;
}

///----------------------------------------------------------------------
///<summary>
///w肳ꂽCfbNẌʒuW擾
///</summary>
///<param name="index">擾ʒuW̃CfbNX</param>
///<returns>w肳ꂽCfbNẌʒuWԋp</returns>
///----------------------------------------------------------------------
Vector3 Mesh::GetPosition( uint32_t index )
{
	assert( index < GetNumPositions() );
	return mPositions[index];
}

///----------------------------------------------------------------------
///<summary>
///w肳ꂽCfbNX̖@xNg擾
///</summary>
///<param name="index">擾@xNg̃CfbNX</param>
///<returns>w肳ꂽCfbNX̖@xNgԋp</returns>
///----------------------------------------------------------------------
Vector3 Mesh::GetNormal( uint32_t index )
{
	assert( index < GetNumNormals() );
	return mNormals[index];
}

///----------------------------------------------------------------------
///<summary>
///w肳ꂽCfbNX̒_J[擾
///</summary>
///<param name="index">擾钸_J[̃CfbNX</param>
///<returns>w肳ꂽCfbNX̒_J[ԋp</returns>
///----------------------------------------------------------------------
UByte4 Mesh::GetColor( uint32_t index )
{
	assert( index < GetNumColors() );
	return mColors[index];
}

///----------------------------------------------------------------------
///<summary>
///w肳ꂽCfbNX̃eNX`W擾
///</summary>
///<param name="index">擾eNX`W̃CfbNX</param>
///<returns>w肳ꂽCfbNX̃eNX`Wԋp</returns>
///----------------------------------------------------------------------
Vector2 Mesh::GetTexCoord( uint32_t index )
{
	assert( index < GetNumTexCoords() );
	return mTexCoords[index];
}

///----------------------------------------------------------------------
///<summary>
///w肳ꂽCfbNX̐ڃxNg擾
///</summary>
///<param name="index">擾ڃxNg̃CfbNX</param>
///<returns>w肳ꂽCfbNX̐ڃxNgԋp</returns>
///----------------------------------------------------------------------
Vector3 Mesh::GetTangent( uint32_t index )
{
	assert( index < GetNumTangents() );
	return mTangents[index];
}

///----------------------------------------------------------------------
///<summary>
///w肳ꂽCfbNX̏]@xNg擾
///</summary>
///<param name="index">擾ڃxNg̃CfbNX</param>
///<returns>w肳ꂽCfbNX̐ڃxNgԋp</returns>
///----------------------------------------------------------------------
Vector3 Mesh::GetBinormal( uint32_t index )
{
	assert( index < GetNumBinormals() );
	return mBinormals[index];
}

///----------------------------------------------------------------------
///<summary>
///w肳ꂽCfbNX̃XLCfbNX擾
///</summary>
///<param name="index">擾XLCfbNX̃CfbNX</param>
///<returns>w肳ꂽCfbNX̃XLCfbNXԋp</returns>
///----------------------------------------------------------------------
Short4 Mesh::GetSkinIndex( uint32_t index )
{
	assert( index < GetNumSkinIndices() );
	return mSkinIndices[index];
}

///----------------------------------------------------------------------
///<summary>
///w肳ꂽCfbNX̃XLEFCg擾
///</summary>
///<param name="index">擾XLEFCg̃CfbNX</param>
///<returns>w肳ꂽCfbNX̃XLEFCgԋp</returns>
///----------------------------------------------------------------------
Vector4 Mesh::GetSkinWeight( uint32_t index )
{
	assert( index < GetNumSkinWeights() );
	return mSkinWeights[index];
}

///----------------------------------------------------------------------
///<summary>
///w肳ꂽCfbNX̒_CfbNX擾
///</summary>
///<param name="index">擾钸_CfbNX̃CfbNX<param>
///<returns>w肳ꂽCfbNX̒_CfbNXԋp</returns>
///----------------------------------------------------------------------
uint32_t Mesh::GetIndex( uint32_t index )
{
	assert( index < GetNumIndices() );
	return mIndices[index];
}

///----------------------------------------------------------------------
///<summary>
///w肳ꂽCfbNX̃{[擾
///</summary>
///<param name="index">擾{[̃CfbNX</param>
///<returns>
///w肳ꂽCfbNX̃{[IBone*^ɕϊĕԋp
///</returns>
///----------------------------------------------------------------------
IBone* Mesh::GetBone( uint32_t index )
{
	assert( index < GetNumBones() );
	return &mBones[index];
}

///----------------------------------------------------------------------
///<summary>
///w肳ꂽCfbNX̃TuZbg擾
///</summary>
///<param name="index">擾TuZbg̃CfbNX</param>
///</returns>
///w肳ꂽCfbNX̃TuZbgIMeshSubset*^ɕϊĕԋp
///</returns>
///----------------------------------------------------------------------
IMeshSubset* Mesh::GetSubset( uint32_t index )
{
	assert( index < GetNumSubsets() );
	return &mSubsets[index];
}

///----------------------------------------------------------------------
///<summary>
///w肳ꂽCfbNX̃}eA擾
///</summary>
///<param name="index">擾TuZbg̃CfbNX</param>
///<returns>
///w肳ꂽCfbNX̃}eAIMaterial*^ɕϊĕԋp
///</returns>
///----------------------------------------------------------------------
IMaterial* Mesh::GetMaterial( uint32_t index )
{
	assert( index < GetNumMaterials() );
	return &mMaterials[index];
}

///----------------------------------------------------------------------
///<summary>
///ʒuW擾
///</summary>
///<returns>ʒuWԋp</returns>
///----------------------------------------------------------------------
uint32_t Mesh::GetNumPositions()
{
	return static_cast< uint32_t >( mPositions.size() );
}

///----------------------------------------------------------------------
///<summary>
///@xNg擾
///</summary>
///<returns>@xNgԋp</returns>
///----------------------------------------------------------------------
uint32_t Mesh::GetNumNormals()
{
	return static_cast< uint32_t >( mNormals.size() );
}

///----------------------------------------------------------------------
///<summary>
///_J[擾
///</summary>
///<returns>_J[ԋp</returns>
///----------------------------------------------------------------------
uint32_t Mesh::GetNumColors()
{
	return static_cast< uint32_t >( mColors.size() );
}

///----------------------------------------------------------------------
///<summary>
///eNX`W擾
///</summary>
///<returns>eNX`Wԋp</returns>
///----------------------------------------------------------------------
uint32_t Mesh::GetNumTexCoords()
{
	return static_cast< uint32_t >( mTexCoords.size() );
}

///----------------------------------------------------------------------
///<summary>
///ڃxNg擾
///</summary>
///<returns>ڃxNgԋp</returns>
///----------------------------------------------------------------------
uint32_t Mesh::GetNumTangents()
{
	return static_cast< uint32_t >( mTangents.size() );
}

///----------------------------------------------------------------------
///<summary>
///]@xNg擾
///</summary>
///<returns>]@xNgԋp</returns>
///----------------------------------------------------------------------
uint32_t Mesh::GetNumBinormals()
{
	return static_cast< uint32_t >( mBinormals.size() );
}

///----------------------------------------------------------------------
///<summary>
///XLCfbNX擾
///</summary>
///<returns>XLCfbNXԋp</returns>
///----------------------------------------------------------------------
uint32_t Mesh::GetNumSkinIndices()
{
	return static_cast< uint32_t >( mSkinIndices.size() );
}

///----------------------------------------------------------------------
///<summary>
///XLEFCg擾
///</summary>
///<returns>XLEFCgԋp</returns>
///----------------------------------------------------------------------
uint32_t Mesh::GetNumSkinWeights()
{
	return static_cast< uint32_t >( mSkinWeights.size() );
}

///----------------------------------------------------------------------
///<summary>
///_CfbNX擾
///</summary>
///<returns>_CfbNXԋp</returns>
///----------------------------------------------------------------------
uint32_t Mesh::GetNumIndices()
{
	return static_cast< uint32_t >( mIndices.size() );
}

///----------------------------------------------------------------------
///<summary>
///{[擾
///</summary>
///<returns>{[ԋp</returns>
///----------------------------------------------------------------------
uint32_t Mesh::GetNumBones()
{
	return static_cast< uint32_t >( mBones.size() );
}

///----------------------------------------------------------------------
///<summary>
///TuZbg擾
///</summary>
///<returns>TuZbgԋp</returns>
///----------------------------------------------------------------------
uint32_t Mesh::GetNumSubsets()
{
	return static_cast< uint32_t >( mSubsets.size() );
}

///----------------------------------------------------------------------
///<summary>
///}eA擾
///</summary>
///<returns>}eAԋp</returns>
///----------------------------------------------------------------------
uint32_t Mesh::GetNumMaterials()
{
	return static_cast< uint32_t >( mMaterials.size() );
}

///----------------------------------------------------------------------
///<summary>
///bVݒ肷
///</summary>
///<param name="name">ݒ肷郁bV</param>
///----------------------------------------------------------------------
void Mesh::SetName( const std::string &name )
{
	mName = name;
}

///----------------------------------------------------------------------
///<summary>
///ʒuWݒ肷
///</summary>
///<param name="positions">ݒ肷ʒuW</param>
///----------------------------------------------------------------------
void Mesh::SetPositions( const std::vector< Vector3 > &positions )
{
	size_t size = positions.size();
	mPositions.clear();
	mPositions.resize( size );
	for( size_t i=0; i<size; i++ )
	{
		mPositions[i] = positions[i];
	}
}

///----------------------------------------------------------------------
///<summary>
///@xNgݒ肷
///</summary>
///<param name="normals">ݒ肷@xNg</param>
///----------------------------------------------------------------------
void Mesh::SetNormals( const std::vector< Vector3 > &normals )
{
	size_t size = normals.size();
	mNormals.clear();
	mNormals.resize( size );
	for( size_t i=0; i<size; i++ )
	{
		mNormals[i] = normals[i];
	}
}

///---------------------------------------------------------------------
///<summary>
///_J[ݒ肷
///</summary>
///<param name="colors">ݒ肷钸_J[</param>
///---------------------------------------------------------------------
void Mesh::SetColors( const std::vector< UByte4 > &colors )
{
	size_t size = colors.size();
	mColors.clear();
	mColors.resize( size );
	for( size_t i=0; i<size; i++ )
	{
		mColors[i] = colors[i];
	}
}

///---------------------------------------------------------------------
///<summary>
///eNX`Wݒ肷
///</summary>
///<param name="texcoords">ݒ肷eNX`W</param>
///---------------------------------------------------------------------
void Mesh::SetTexCoords( const std::vector< Vector2 > &texcoords )
{
	size_t size = texcoords.size();
	mTexCoords.clear();
	mTexCoords.resize( size );
	for( size_t i=0; i<size; i++ )
	{
		mTexCoords[i] = texcoords[i];
	}
}

///---------------------------------------------------------------------
///<summary>
///ڃxNgݒ肷
///</summary>
///<param name="tangents">ݒ肷ڃxNg</param>
///---------------------------------------------------------------------
void Mesh::SetTangents( const std::vector< Vector3 > &tangents )
{
	size_t size = tangents.size();
	mTangents.clear();
	mTangents.resize( size );
	for( size_t i=0; i<size; i++ )
	{
		mTangents[i] = tangents[i];
	}
}

///---------------------------------------------------------------------
///<summary>
///]@xNgݒ肷
///</summary>
///<param name="binormals">ݒ肷]@xNg</param>
///---------------------------------------------------------------------
void Mesh::SetBinormals( const std::vector< Vector3 > &binormals )
{
	size_t size = binormals.size();
	mBinormals.clear();
	mBinormals.resize( size );
	for( size_t i=0; i<size; i++ )
	{
		mBinormals[i] = binormals[i];
	}
}

///---------------------------------------------------------------------
///<summary>
///XLCfbNXݒ肷
///</summary>
///<param name="skin_indices">ݒ肷XLCfbNX</param>
///---------------------------------------------------------------------
void Mesh::SetSkinIndices( const std::vector< Short4 > &skin_indices )
{
	size_t size = skin_indices.size();
	mSkinIndices.clear();
	mSkinIndices.resize( size );
	for( size_t i=0; i<size; i++ )
	{
		mSkinIndices[i] = skin_indices[i];
	}
}

///---------------------------------------------------------------------
///<summary>
///XLEFCgݒ肷
///</summary>
///<param name="skin_weights">ݒ肷XLEFCg</param>
///---------------------------------------------------------------------
void Mesh::SetSkinWeights( const std::vector< Vector4 > &skin_weights )
{
    size_t size = skin_weights.size();
	mSkinWeights.clear();
	mSkinWeights.resize( size );
	for( size_t i=0; i<size; i++ )
	{
		mSkinWeights[i] = skin_weights[i];
	}
}

///---------------------------------------------------------------------
///<summary>
///_CfbNXݒ肷
///</summary>
///<param name="indices">ݒ肷钸_CfbNX</param>
///---------------------------------------------------------------------
void Mesh::SetIndices( const std::vector< uint32_t > &indices )
{
    size_t size = indices.size();
    mIndices.clear();
    mIndices.resize( size );
    for( size_t i=0; i<size; i++ )
    {
        mIndices[i] = indices[i];
    }
}

///---------------------------------------------------------------------
///<summary>
///{[ݒ肷
///</summary>
///<param name="bones">ݒ肷{[</param>
///---------------------------------------------------------------------
void Mesh::SetBones( const std::vector< Bone > &bones )
{
    size_t size = bones.size();
    mBones.clear();
    mBones.resize( size );
    for( size_t i=0; i<size; i++ )
    {
        mBones[i] = bones[i];
    }
}

///---------------------------------------------------------------------
///<summary>
///TuZbgݒ肷
///</summary>
///<param name="subsets">ݒ肷TuZbg</param>
///---------------------------------------------------------------------
void Mesh::SetSubsets( const std::vector< MeshSubset > &subsets )
{
    size_t size = subsets.size();
    mSubsets.clear();
    mSubsets.resize( size );
    for( size_t i=0; i<size; i++ )
    {
        mSubsets[i] = subsets[i];
    }
}

///---------------------------------------------------------------------
///<summary>
///}eAݒ肷
///</summary>
///<param name="materials">ݒ肷}eA</param>
///---------------------------------------------------------------------
void Mesh::SetMaterials( const std::vector< Material> &materials )
{
    size_t size = materials.size();
    mMaterials.clear();
    mMaterials.resize( size );
    for( size_t i=0; i<size; i++ )
    {
        mMaterials[i] = materials[i];
    }
}

///---------------------------------------------------------------------
///<summary>
///@xNgvZ
///</summary>
///---------------------------------------------------------------------
void Mesh::ComputeNormals()
{
	/*  */
}

///---------------------------------------------------------------------
///<summary>
///ڃxNgvZ
///</summary>
///---------------------------------------------------------------------
void Mesh::ComputeTangents()
{
	/*  */
}

///---------------------------------------------------------------------
///<summary>
///]@xNgvZ
///</summary>
///---------------------------------------------------------------------
void Mesh::ComputeBinormals()
{
	/*  */
}

Mesh& Mesh::operator = ( const Mesh &mesh )
{
	SetName( mesh.mName );
	SetPositions( mesh.mPositions );
	SetNormals( mesh.mNormals );
	SetColors( mesh.mColors );
	SetTexCoords( mesh.mTexCoords );
	SetTangents( mesh.mTangents );
	SetBinormals( mesh.mBinormals );
	SetSkinIndices( mesh.mSkinIndices );
	SetSkinWeights( mesh.mSkinWeights );
    SetIndices( mesh.mIndices );
    SetBones( mesh.mBones );
    SetSubsets( mesh.mSubsets );
    SetMaterials( mesh.mMaterials );
	return (*this);
}


/////////////////////////////////////////////////////////////////////////
// KeyPose class
/////////////////////////////////////////////////////////////////////////

///----------------------------------------------------------------------
///<summary>
///RXgN^
///</summary>
///----------------------------------------------------------------------
KeyPose::KeyPose()
{
    mTime = 0;
    mScale = Vector3( 1.0f, 1.0f, 1.0f );
    mRotate = Quaternion();
    mTranslate = Vector3( 0.0f, 0.0f, 0.0f );
    mTransform.Identity();
}

///----------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///----------------------------------------------------------------------
KeyPose::~KeyPose()
{
    /* DO_NOTHING */
}

///----------------------------------------------------------------------
///<summary>
///Ԃ擾
///</summary>
///<returns>Ԃԋp</returns>
///----------------------------------------------------------------------
uint32_t KeyPose::GetTime()
{
    return mTime;
}

///----------------------------------------------------------------------
///<summary>
///gɗpxNg擾
///</summary>
///<returns>gɗpxNgԋp</returns>
///----------------------------------------------------------------------
Vector3 KeyPose::GetScale()
{
    return mScale;
}

///----------------------------------------------------------------------
///<summary>
///]ɗpl擾
///</summary>
///<returns>]ɗplԋp</returns>
///----------------------------------------------------------------------
Quaternion KeyPose::GetRotate()
{
    return mRotate;
}

///----------------------------------------------------------------------
///<summary>
///sړɗpxNg擾
///</summary>
///<returns>sړɗpxNgԋp</returns>
///----------------------------------------------------------------------
Vector3 KeyPose::GetTranslate()
{
    return mTranslate;
}

///----------------------------------------------------------------------
///<summary>
///Aj[Vɗpϊs擾
///</summary>
///<returns>Aj[Vɗpϊsԋp</returns>
///----------------------------------------------------------------------
Matrix KeyPose::GetTransform()
{
    return mTransform;
}

///----------------------------------------------------------------------
///<summary>
///Ԃݒ肷
///</summary>
///<param name="time">ݒ肷鎞</param>
///----------------------------------------------------------------------
void KeyPose::SetTime( const uint32_t &time )
{
    mTime = time;
}

///----------------------------------------------------------------------
///<summary>
///gɗpxNgݒ肷
///</summary>
///<param name="scale">ݒ肷xNg</param>
///----------------------------------------------------------------------
void KeyPose::SetScale( const Vector3 &scale )
{
    mScale = scale;
}

///----------------------------------------------------------------------
///<summary>
///]ɗplݒ肷
///</summary>
///<param name="rotate">ݒ肷l</param>
///----------------------------------------------------------------------
void KeyPose::SetRotate( const Quaternion &rotate )
{
    mRotate = rotate;
}

///----------------------------------------------------------------------
///<summary>
///sړɗpxNgݒ肷
///</summary>
///<param name="translate">ݒ肷xNg</param>
///----------------------------------------------------------------------
void KeyPose::SetTranslate( const Vector3 &translate )
{
    mTranslate = translate;
}

///----------------------------------------------------------------------
///<summary>
///Aj[Vɗpϊsݒ肷
///</summary>
///<param name="transform">ݒ肷ϊs</param>
///----------------------------------------------------------------------
void KeyPose::SetTransform( const Matrix &transform )
{
    mTransform = transform;
}

KeyPose& KeyPose::operator = ( const KeyPose &keyPose )
{
    mTime = keyPose.mTime;
    mScale = keyPose.mScale;
    mRotate = keyPose.mRotate;
    mTranslate = keyPose.mTranslate;
    mTransform = keyPose.mTransform;

    return (*this);
}

/////////////////////////////////////////////////////////////////////////
// Animation class
/////////////////////////////////////////////////////////////////////////

///----------------------------------------------------------------------
///<summary>
///RXgN^
///</summary>
///----------------------------------------------------------------------
Animation::Animation()
{
    mName.clear();
    mKeys.clear();
    mBoneIndex = 0;
}

///----------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///----------------------------------------------------------------------
Animation::~Animation()
{
    Release();
}

///----------------------------------------------------------------------
///<summary>
///
///</summary>
///----------------------------------------------------------------------
void Animation::Release()
{
    mName.clear();
    mKeys.clear();
}

///----------------------------------------------------------------------
///<summary>
///O擾
///</summary>
///<returns>Oԋp</returns>
///----------------------------------------------------------------------
std::string Animation::GetName()
{
    return mName;
}

///----------------------------------------------------------------------
///<summary>
///̃CfbNX擾
///</summary>
///<returns>̃CfbNXԋp</returns>
///----------------------------------------------------------------------
uint32_t Animation::GetBoneIndex()
{
    return mBoneIndex;
}

///----------------------------------------------------------------------
///<summary>
///L[|[Y擾
///</summary>
///<returns>L[|[Yԋp</returns>
///----------------------------------------------------------------------
uint32_t Animation::GetNumKeys()
{
    return static_cast< uint32_t >( mKeys.size() );
}

///----------------------------------------------------------------------
///<summary>
///w肳ꂽL[|[Y擾
///</summary>
///<param name="index">擾L[|[ỸCfbNX</param>
///<returns>w肳ꂽCfbNX̃L[|[Y̒lԋp</returns>
///----------------------------------------------------------------------
IKeyPose* Animation::GetKey( uint32_t index )
{
    assert( index < GetNumKeys() );
    return &mKeys[index];
}

///----------------------------------------------------------------------
///<summary>
///L[|[Y擾
///</summary>
///<returns>L[|[Yԋp</returns>
///----------------------------------------------------------------------
std::vector< IKeyPose* > Animation::GetKeys()
{
    std::vector< IKeyPose* > result;
    result.resize( GetNumKeys() );
    for( uint32_t i=0; i<GetNumKeys(); ++i )
    {
        result[i] = &mKeys[i];
    }
    return result;
}

///----------------------------------------------------------------------
///<summary>
///Oݒ肷
///</summary>
///<param name="name">ݒ肷閼O</param>
///----------------------------------------------------------------------
void Animation::SetName( const std::string &name )
{
    mName = name;
}

///----------------------------------------------------------------------
///<summary>
///̃CfbNXݒ肷
///</summary>
///<param name="index"></param>
///----------------------------------------------------------------------
void Animation::SetBoneIndex( const uint32_t &index )
{
    mBoneIndex = index;
}

///----------------------------------------------------------------------
///<summary>
///L[|[Yݒ肷
///</summary>
///<param name="keyposes">ݒ肷L[|[Y</param>
///----------------------------------------------------------------------
void Animation::SetKeys( const std::vector< KeyPose > &keys )
{
    mKeys.resize( keys.size() );
    for( size_t i=0; i<keys.size(); ++i )
    {
        mKeys[i] = keys[i];
    }
}

///----------------------------------------------------------------------
///<summary>
///w肳ꂽCfbNX̃L[|[Yݒ肷
///</summary>
///<param name="index">ݒ肷CfbNX</param>
///<param name="pose">ݒ肷L[|[Y</param>
///----------------------------------------------------------------------
void Animation::SetKey( uint32_t index, const KeyPose &pose )
{
    assert( index < GetNumKeys() );
    mKeys[index] = pose;
}

Animation& Animation::operator = ( const Animation &animation )
{
    mName = animation.mName;
    mBoneIndex = animation.mBoneIndex;
    mKeys.clear();
    mKeys.resize( animation.mKeys.size() );
    for( size_t i=0; i<animation.mKeys.size(); ++i )
    {
        mKeys[i] = animation.mKeys[i];
    }

    return (*this);
}

/////////////////////////////////////////////////////////////////////////
// AnimationClip class
/////////////////////////////////////////////////////////////////////////

///----------------------------------------------------------------------
///<summary>
///RXgN^
///</summary>
///----------------------------------------------------------------------
AnimationClip::AnimationClip()
{
    mName.clear();
    mFrames.clear();
}

///----------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///----------------------------------------------------------------------
AnimationClip::~AnimationClip()
{
   Release();
}

///---------------------------------------------------------------------
///<summary>
///
///</summary>
///---------------------------------------------------------------------
void AnimationClip::Release()
{ 
    mName.clear();
    mFrames.clear();
}

///---------------------------------------------------------------------
///<summary>
///Aj[VNbv擾
///</summary>
///<returns>Aj[VNbvԋp</returns>
///---------------------------------------------------------------------
std::string AnimationClip::GetName()
{ return mName; }

///---------------------------------------------------------------------
///<summary>
///w肳ꂽCfbNX̃{[Aj[Vf[^擾
///</summary>
///<param name="index">擾CfbNX</param>
///<returns>
///w肳ꂽCfbNX̃{[Aj[Vf[^ԋp
///</returns>
///---------------------------------------------------------------------
IAnimation* AnimationClip::GetFrame( uint32_t index )
{
    assert( index < GetNumFrames() );
    return &mFrames[index];
}

///----------------------------------------------------------------------
///<summary>
///{[Aj[Vf[^擾
///</summary>
///<returns>{[Aj[Vf[^ԋp</returns>
///----------------------------------------------------------------------
std::vector< IAnimation* > AnimationClip::GetFrames()
{
    std::vector< IAnimation* > result;
    result.resize( GetNumFrames() );
    for( uint32_t i=0; i<GetNumFrames(); ++i )
    {
        result[i] = &mFrames[i];
    }
    return result;
}

///----------------------------------------------------------------------
///<summary>
///{[Aj[Vf[^擾
///</summary>
///<returns>{[Aj[Vf[^ԋp</returns>
///----------------------------------------------------------------------
uint32_t AnimationClip::GetNumFrames()
{
    return static_cast< uint32_t >( mFrames.size() );
}

///----------------------------------------------------------------------
///<summary>
///Aj[VpԂ擾
///</summary>
///<returns>Aj[VpԂԋp</returns>
///----------------------------------------------------------------------
float AnimationClip::GetDuration()
{
    return mDuration;
}

///---------------------------------------------------------------------
///<summary>
///Aj[VNbvݒ肷
///</summary>
///<param name="name">ݒ肷Aj[VNbv</param>
///---------------------------------------------------------------------
void AnimationClip::SetName( const std::string &name )
{ mName = name; }


///---------------------------------------------------------------------
///<summary>
///w肳ꂽCfbNX̃{[Aj[Vf[^ݒ肷
///</summary>
///<param name="index">ݒ肷CfbNX</param>
///<param name="frame">ݒ肷f[^</param>
///---------------------------------------------------------------------
void AnimationClip::SetFrame( uint32_t index, Animation &frame )
{
    assert( index < GetNumFrames() );
    mFrames[index] = frame;
}

///---------------------------------------------------------------------
///<summary>
///{[Aj[Vf[^ݒ肷
///</summary>
///<param name="frames">ݒ肷f[^</param>
///---------------------------------------------------------------------
void AnimationClip::SetFrames( const std::vector< Animation > &frames )
{
    mFrames.clear();
    mFrames.resize( frames.size() );
    for( size_t i=0; i<frames.size(); ++i )
    {
        mFrames[i] = frames[i];
    }
}

///----------------------------------------------------------------------
///<summary>
///Aj[VpԂݒ肷
///</summary>
///<param name="duration">ݒ肷鎞ԕ</param>
///----------------------------------------------------------------------
void AnimationClip::SetDuration( const float &duration )
{
    mDuration = duration;
}

AnimationClip& AnimationClip::operator = ( const AnimationClip &clip )
{
    mName = clip.mName;
    mDuration = clip.mDuration;
    mFrames.clear();
    mFrames.resize( clip.mFrames.size() );
    for( size_t i=0; i<clip.mFrames.size(); ++i )
    {
        mFrames[i] = clip.mFrames[i];
    }
   
    return (*this);
}

/////////////////////////////////////////////////////////////////////////
// AnimationPlayer class
/////////////////////////////////////////////////////////////////////////

///---------------------------------------------------------------------
///<summary>
///RXgN^
///</summary>
///<param name="bones">Aj[V郁bṼ{[</param>
///---------------------------------------------------------------------
AnimationPlayer::AnimationPlayer( const std::vector< IBone* > &bones )
{
    mCurTime = 0.0f;
    mpClip = null;
    
    size_t size = bones.size();
    mBones.clear();
    mBones.resize( size );
    for( size_t i=0; i<size; i++ )
    {
        mBones[i] = bones[i];
    }

    mBoneTransforms.clear();
    mWorldTransforms.clear();
    mSkinTransforms.clear();

    mBoneTransforms.resize( size );
    mWorldTransforms.resize( size );
    mSkinTransforms.resize( size );  
}

///---------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///---------------------------------------------------------------------
AnimationPlayer::~AnimationPlayer()
{
    /* DO NOTHING */
}

///---------------------------------------------------------------------
///<summary>
///
///</summary>
///---------------------------------------------------------------------
void AnimationPlayer::Release()
{
    mpClip = null;
    mBones.clear();
    mBoneTransforms.clear();
    mWorldTransforms.clear();
    mSkinTransforms.clear();
}

///---------------------------------------------------------------------
///<summary>
///Aj[VNbvݒ肷
///</summary>
///<param name="pclip">ݒ肷Aj[VNbv</param>
///---------------------------------------------------------------------
void AnimationPlayer::SetClip( IAnimationClip* pclip )
{ 
    mpClip = pclip;
    mCurTime = 0.0f;

    for( size_t i=0; i<mBones.size(); i++ )
    {
        mBoneTransforms[i] = mBones[i]->GetPoseMatrix();
    }
}

///---------------------------------------------------------------------
///<summary>
///{[ϊsXV
///</summary>
///<param name="time"></param>
///<param name="relativeToCurTime></param>
///---------------------------------------------------------------------
void AnimationPlayer::UpdateBoneTransform( float time, bool relativeToCurTime )
{
    if ( mpClip == null )
    {
        ELOG( "Error : AnimationPlayer::UpdateBoneTransform() mpClip == null" );
        assert( false );
    }

    if ( relativeToCurTime )
    {
        //@Ԃi߂
        time += mCurTime;

        //@pԂ傫ȂC߂ɖ߂
        while( time >= mpClip->GetDuration() )
        {
            time -= mpClip->GetDuration();
        }
    }
    //@Ԃ͈̔̓`FbN
    if ( (time < 0.0f) || (time >= mpClip->GetDuration()) )
    {
        DLOG( "Error : AnimationPlayer::UpdateBoneTransform() time value range is invalid." );
        assert( false );
    }

    //@ݎԂȂC
    if ( time < mCurTime )
    {
        for( size_t i=0; i<mBones.size(); i++ )
        {
            mBoneTransforms[i] = mBones[i]->GetPoseMatrix();
        }
    }

    //@ݎԂXV
    mCurTime = time;

    uint32_t num_frames = mpClip->GetNumFrames();
    for( uint32_t i=0; i<num_frames; ++i )
    {
        //@Aj[Vf[^擾
        IAnimation* animation = mpClip->GetFrame( i );

        uint32_t index1 = 0;
        uint32_t index2 = 0;
        //@ݎԂɋ߂t[
        for( uint32_t j=0; j<animation->GetNumKeys(); ++j )
        {
            if ( animation->GetKey( j )->GetTime() > mCurTime )
            {
                index1 = j;
                index2 = ( j > 0 ) ? ( j - 1 ) : j;
                break;
            }
        }

		//@̃CfbNX擾
        uint32_t bone_index = animation->GetBoneIndex();
		assert( bone_index >= 0 );

		//@̃CfbNX{[Aj[Vɗpϊs擾
		Matrix mat1 = animation->GetKey( index1 )->GetTransform();
		Matrix mat2 = animation->GetKey( index2 )->GetTransform();

		//@̕ϊs߂
		mBoneTransforms[ bone_index ] = mat1 + mat2;
    }
}

///---------------------------------------------------------------------
///<summary>
///[hsXV
///</summary>
///<param name="rootMatrix">[gm[h̃[hs</param>
///---------------------------------------------------------------------
void AnimationPlayer::UpdateWorldTransform( const Matrix &rootMatrix )
{
	//@e̕ϊs߂
    mWorldTransforms[0] = mBoneTransforms[0] * rootMatrix;

	//@q̕ϊs߂
    for( size_t i=1; i<mBones.size(); i++ )
    {
        int parent = mBones[i]->GetParentIndex();
		assert( parent >= 0 );

		mWorldTransforms[i] = mBoneTransforms[i] * mWorldTransforms[parent];
    }
}

///---------------------------------------------------------------------
///<summary>
///XLjOsXV
///</summary>
///---------------------------------------------------------------------
void AnimationPlayer::UpdateSkinTransform()
{
    for( size_t i=0; i<mBones.size(); i++ )
    {
		//@XLjOs߂
         mSkinTransforms[i] = mBones[i]->GetBindMatrix() * mWorldTransforms[i];
    }
}

///---------------------------------------------------------------------
///<summary>
///Aj[VXV
///</summary>
///---------------------------------------------------------------------
void AnimationPlayer::Update( float time, bool relativeToCurTime, const Matrix &rootTransform )
{
	//@𓮂
    UpdateBoneTransform( time, relativeToCurTime );

	//@[hϊsXV
    UpdateWorldTransform( rootTransform );

	//@XLjOsXV
    UpdateSkinTransform();
}

///--------------------------------------------------------------------
///<summary>
///XLjOs擾
///</summary>
///<returns>XLjOsԋp</returns>
///--------------------------------------------------------------------
std::vector< Matrix > AnimationPlayer::GetSkinTransform()
{ return mSkinTransforms; }


/////////////////////////////////////////////////////////////////////////
// Model class
/////////////////////////////////////////////////////////////////////////

///---------------------------------------------------------------------
///<summary>
///RXgN^
///</summary>
///---------------------------------------------------------------------
Model::Model()
{
	mName.clear();
	mMeshes.clear();
	mClips.clear();
}

///---------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///---------------------------------------------------------------------
Model::~Model()
{
	/* DO_NOTHING */
}

///---------------------------------------------------------------------
///<summary>
///
///</summary>
///---------------------------------------------------------------------
void Model::Release()
{
	for( size_t i=0; i<mMeshes.size(); i++ )
	{
		mMeshes[i].Release();
	}
	for( size_t i=0; i<mClips.size(); i++ )
	{
		mClips[i].Release();
	}

	mName.clear();
	mMeshes.clear();
	mClips.clear();
}

///---------------------------------------------------------------------
///<summary>
///f擾
///</summary>
///---------------------------------------------------------------------
std::string Model::GetName()
{
	return mName;
}

///---------------------------------------------------------------------
///<summary>
///bV擾
///</summary>
///<returns>IMesh*^ɕϊbVReiԋp</returns>
///---------------------------------------------------------------------
std::vector< IMesh* > Model::GetMeshes()
{
	std::vector< IMesh* > result;
	result.resize( mMeshes.size() );
	for( size_t i=0; i<mMeshes.size(); i++ )
	{
		result[i] = &mMeshes[i];
	}
	return result;
}

///---------------------------------------------------------------------
///<summary>
///Aj[VNbv擾
///</summary>
///<returns>
///IAnimationClip*^ɕϊAj[VNbvReiԋp
///</returns>
///---------------------------------------------------------------------
std::vector< IAnimationClip* > Model::GetClips()
{
	std::vector< IAnimationClip* > result;
	result.resize( mClips.size() );
	for( size_t i=0; i<mClips.size(); i++ )
	{
		result[i] = &mClips[i];
	}
	return result;
}

///---------------------------------------------------------------------
///<summary>
///w肳ꂽCfbNX̃bV擾
///</summary>
///<param name="index">擾郁bṼCfbNX</param>
///<returns>w肳ꂽCfbNX̃bVԋp</returns>
///---------------------------------------------------------------------
IMesh* Model::GetMesh( uint32_t index )
{
	assert( index < GetNumMeshes() );
	return &mMeshes[index];
}

///---------------------------------------------------------------------
///<summary>
///w肳ꂽCfbNX̃Aj[VNbv擾
///</summary>
///<param name="index">
///擾Aj[VNbṽCfbNX
///</param>
///<returns>
///w肳ꂽCfbNX̃Aj[VNbvԋp
///<returns>
///---------------------------------------------------------------------
IAnimationClip* Model::GetClip( uint32_t index )
{
	assert( index < GetNumClips() );
	return &mClips[index];
}

///---------------------------------------------------------------------
///<summary>
///bV擾
///</summary>
///<returns>bVԋp</returns>
///---------------------------------------------------------------------
uint32_t Model::GetNumMeshes()
{
	return static_cast< uint32_t >( mMeshes.size() );
}

///---------------------------------------------------------------------
///<summary>
///Aj[VNbv擾
///</summary>
///<returns>Aj[VNbvԋp</returns>
///---------------------------------------------------------------------
uint32_t Model::GetNumClips()
{
	return static_cast< uint32_t >( mClips.size() );
}

///---------------------------------------------------------------------
///<summary>
///fݒ肷
///</summary>
///<param name="name">ݒ肷郂f</param>
///---------------------------------------------------------------------
void Model::SetName( const std::string &name )
{
	mName = name;
}

///---------------------------------------------------------------------
///<summary>
///bVݒ肷
///</summary>
///<param name="meshes">ݒ肷郁bV</param>
///---------------------------------------------------------------------
void Model::SetMeshes( const std::vector< Mesh > &meshes )
{
	mMeshes.clear();
	mMeshes.resize( meshes.size() );
	for( size_t i=0; i<meshes.size(); i++ )
	{
		mMeshes[i] = meshes[i];
	}
}

///--------------------------------------------------------------------
///<summary>
///Aj[VNbvݒ肷
///</summary>
///<param name="clips">ݒ肷Aj[VNbv</param>
///--------------------------------------------------------------------
void Model::SetClips( const std::vector< AnimationClip > &clips )
{
	mClips.clear();
	mClips.resize( clips.size() );
	for( size_t i=0; i<clips.size(); i++ )
	{
		mClips[i] = clips[i];
	}
}

Model& Model::operator = ( const Model &model )
{
	SetName( model.mName );
	SetMeshes( model.mMeshes );
	SetClips( model.mClips );
	return (*this);
}

/////////////////////////////////////////////////////////////////////////
// ModelLoaderBase class
/////////////////////////////////////////////////////////////////////////

///----------------------------------------------------------------------
///<summary>
///RXgN^
///</summary>
///----------------------------------------------------------------------
ModelLoaderBase::ModelLoaderBase( uint32_t size ) : mToken( size )
{
    mFileName.clear();
    mpBuf = null;
    mpFile = null;
    mIsLoading = false;
    mBufSize = 0;
    mCurSize = 0;
}

///----------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///----------------------------------------------------------------------
ModelLoaderBase::~ModelLoaderBase()
{
	/* DO NOTHING */
}

///----------------------------------------------------------------------
///<summary>
///
///</summary>
///----------------------------------------------------------------------
void ModelLoaderBase::Release()
{
    mFileName.clear();
    SAFE_DELETE_ARRAY( mpBuf );
	mModel.Release();
    mpFile = null;
    mIsLoading = false;
    mBufSize = 0;
    mCurSize = 0;
    mToken.Release();
}

///----------------------------------------------------------------------
///<summary>
///f擾
///</summary>
///----------------------------------------------------------------------
IModel* ModelLoaderBase::GetModel()
{
	Model* pModel = new Model();
	(*pModel) = mModel;
    return pModel;
}

///----------------------------------------------------------------------
///<summary>
///t@C烂f[h
///</summary>
///<param name="filename">[ht@C</param>
///----------------------------------------------------------------------
bool ModelLoaderBase::Load( const char *filename )
{
	//@t@CJ
    mpFile = fopen( filename, "rb" );
    if ( !mpFile )
    {
        ELOG( "Error : file open failed - %s", filename );
        return false;
    }

	//@t@CRs[
    mFileName = std::string( filename );

	//@TCY擾
    fseek( mpFile, 0, SEEK_END );
    mBufSize = static_cast< uint32_t >( ftell( mpFile ) );

	//@t@C̐擪ɖ߂
    fseek( mpFile, 0, SEEK_SET );

	//@mۂ
    try 
    {
        mpBuf = new char [mBufSize+1];
    }
    catch( std::bad_alloc ba )
    {
        ELOG( "Error : memory allocate failed. reason = %s", ba.what() );
        return false;
    }

	//@[NA
    memset( mpBuf, 0, mBufSize ); 
    mpBuf[mBufSize] = '\0';

	//@TCYݒ
    mCurSize = 0;
    mRestSize = mBufSize;

    DLOG( "Info : Now Loading - %s", filename );

    return true;
}

///----------------------------------------------------------------------
///<summary>
///[hǂ肷
///</summary>
///<returns>
///[hȂtrueԋp.
///[hI܂̓[hłȂꍇfalseԋp
///</returns>
///----------------------------------------------------------------------
bool ModelLoaderBase::IsLoading()
{
    mIsLoading = true;

	//@t@CJĂȂꍇ̓[hłȂ̂ŏI
    if ( !mpFile )
    {
        ELOG( "Error : file not opened." );
        mIsLoading = false;
        return mIsLoading;
    }

    const int READ_SIZE = 131072; // 128[byte] * 1024 = 131072[byte]

	//@ǂݍ݃TCY
    uint32_t size = ( mRestSize > READ_SIZE ) ? READ_SIZE : mRestSize;

	//@t@Cǂݍ
    fread( &mpBuf[ mCurSize ], sizeof(char), size, mpFile );

	//@ǂݍ񂾕TCYZ
    mCurSize += size;
    mRestSize -= size;
    
	//@SēǂݍݏIꍇ
	if ( mRestSize == 0 )
    {
		//@t@C
        fclose( mpFile );
        DLOG( "Info : Load Finished!" );
        mIsLoading = false;

		//@[hI
        return mIsLoading;
    }

	//@[h
    return mIsLoading;
}

///----------------------------------------------------------------------
///<summary>
///t@Cgq擾
///</summary>
///<return>gqԋp</returns>
///----------------------------------------------------------------------
std::string ModelLoaderBase::GetExt()
{
    int32_t index = mFileName.rfind( "." );
    if ( index > 0 )
    {
       std::string ext = mFileName.substr( index );
       DLOG( "Info : MeshLoaderBase::GetExt() ret=%s", ext.c_str() );
       return ext;
    }
    ELOG( "Error : MeshLoaderBase::GetExt() not find extention." );
    return std::string("\0");
}

///----------------------------------------------------------------------
///<summary>
///t@CfBNg擾
///</summary>
///<returns>fBNgԋp</returns>
///----------------------------------------------------------------------
std::string ModelLoaderBase::GetDir()
{
    std::string dir;
    int32_t index = mFileName.rfind( "\\" );
    if ( index > 0 )
    {
        dir.resize( index );
        for( int32_t i=0; i<index; i++ )
        {
            dir[i] = mFileName[i];
        }
        DLOG( "Info : MeshLoaderBase::GetDir() ret=%s", dir.c_str() );
        return dir;
    }

    index = mFileName.rfind( "/" );
    if ( index > 0 )
    {
        dir.resize( index );
        for( int32_t i=0; i<index; i++ )
        {
            dir[i] = mFileName[i];
        }
        DLOG( "Info : MeshLoaderBase::GetDir() ret=%s", dir.c_str() );
        return dir;
    }
    ELOG( "Error : MeshLoaderBase::GetDir() not find \'/\' or \'\\\'" );
    return std::string("\0");
}


///////////////////////////////////////////////////////////////////////////
// ModelLoaderFactory
///////////////////////////////////////////////////////////////////////////

///------------------------------------------------------------------------
///<summary>
///[_[쐬
///</summary>
///<param name="type">쐬郍[_[̃^Cv</param>
///<param name="buf_size">g[Ñobt@TCY</param>
///<retrns>w肳ꂽ^Cv쐬[_ԋp</returns>
///------------------------------------------------------------------------
IModelLoader* ModelLoaderFactory::Create( int32_t type, uint32_t token_size )
{
    switch( type )
    {
	    case MODEL_TYPE_X:	{ return new ModelLoaderX( token_size ); }
        default:            { return null; }
    }
}

/////////////////////////////////////////////////////////////////////////
// AnimationPlayerFactory class
/////////////////////////////////////////////////////////////////////////

///--------------------------------------------------------------------
///<summary>
///Aj[VvC[𐶐
///</summary>
///<param name="bones">Aj[V郂f̍f[^</param>
///<returns>Aj[Vv[[ԋp</returns>
///--------------------------------------------------------------------
IAnimationPlayer* AnimationPlayerFactory::Create( std::vector< IBone* > bones )
{
    AnimationPlayer* pPlayer = null;
    try 
    {
        pPlayer = new AnimationPlayer( bones );
    }
    catch( std::bad_alloc ba )
    {
        ELOG( "Error : Memory Allocate Failed. reason=%s", ba.what() );
    }
    return pPlayer;
}


} // namespace Graphics
} // namespace Asura
