//-----------------------------------------------------------------------
// File : AsuraTokenizer.cpp
// Desc : Tokenizer
// Copyright(c) Project Asura All right reserved.
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
// History
//-----------------------------------------------------------------------
// <---> 2010/11/13 [Pocol] VK쐬.
// <End>


//-----------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------
#include "asTokenizer.h"


namespace Asura {
namespace Util {

///-----------------------------------------------------------------------
///<summary>
///RXgN^
///</summary>
///<param name="size">g[ÑTCY</param>
///-----------------------------------------------------------------------
Tokenizer::Tokenizer( uint32_t size )
{
	mBuffer = null;
	mSeparator.clear();
	mCutOff.clear();
	try 
	{ 
		mToken = new char [size];
	} 
	catch( std::bad_alloc ba )
	{
		std::cerr << "Error : " << ba.what() << "\n";
	}
}

///-----------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///-----------------------------------------------------------------------
Tokenizer::~Tokenizer()
{
	mSeparator.clear();
	mCutOff.clear();
	SAFE_DELETE( mToken );
}

///----------------------------------------------------------------------
///<summary>
///؂蕶ݒ肷
///</summary>
///<param name="separator">؂蕶ƂĐݒ肷镶</param>
///----------------------------------------------------------------------
void Tokenizer::SetSeparator( const char *separator )
{
	mSeparator = std::string( separator );
}

///-----------------------------------------------------------------------
///<summary>
///؂oݒ肷
///</summary>
///<param name="cutfoff">P̃g[NƂĐ؂o</param>
///-----------------------------------------------------------------------
void Tokenizer::SetCutOff( const char *cutoff )
{
	mCutOff = std::string( cutoff );
}

///----------------------------------------------------------------------
///<summary>
///obt@ւ̃|C^ݒ肷
///</summary>
///<param name="buffer">obt@ւ̃|C^</param>
///----------------------------------------------------------------------
void Tokenizer::SetBuffer( char *buffer )
{
	mBuffer = buffer;
}

///----------------------------------------------------------------------
///<summary>
///
///</summary>
///----------------------------------------------------------------------
void Tokenizer::Release()
{
	mSeparator.clear();
	mCutOff.clear();
	SAFE_DELETE( mToken );
	mBuffer = null;
}

///----------------------------------------------------------------------
///<summary>
///̃g[N擾
///</summary>
///----------------------------------------------------------------------
void Tokenizer::Next()
{
	char *p = mBuffer;
	char *q = mToken;

	// ؂蕶̓XLbv
	while ( (*p) != '\0' && strchr( mSeparator.c_str(), (*p) ) )
	{
		p++;
	}

	// ؂oƃqbg邩
	if ( strchr( mCutOff.c_str(), (*p) ) )
	{
		//؂oƃqbgCP̃g[NƂ
		(*(q++)) = (*(p++));
	}
	else
	{
		//؂蕶܂͐؂oȊOȂg[NƂ
		std::string split = mSeparator + mCutOff;
		while( (*p) != '\0' && !strchr( split.c_str(), (*p) ) )
		{
			(*(q++)) = (*(p++));
		}
	}

	//oobt@i߂
	mBuffer = p;

	//ƂĕԂ߂NULLI[
	(*q) = '\0';
}

///-----------------------------------------------------------------------
///<summary>
///w肳ꂽƃg[NvĂ邩肷
///</summary>
///<param name="token">g[NƔr镶</param>
///<returns>
///g[NƈvĂtrueԋpAłȂȂfalseԋp
///</returns>
///-----------------------------------------------------------------------
bool Tokenizer::Check( const char *token )
{
	bool result = false;
	if ( strcmp( mToken, token ) == 0 )
	{
		result = true;
	}
	return result;
}

///-----------------------------------------------------------------------
///<summary>
///I
///</summary>
///<returns>
///obt@NULLI[ɂȂꍇAtrueԋp.
///obt@ւ̃|C^NULL̏ꍇCtrueԋp.
///LȊȌꍇ́Cfalseԋp.
///</returns>
///-----------------------------------------------------------------------
bool Tokenizer::IsEnd()
{
	bool result = false;
	if ( (*mBuffer) == '\0' || mBuffer == null )
	{
		result = true;
	}
	return result;
}

///-----------------------------------------------------------------------
///<summary>
///g[Nchar^Ƃĕԋp
///</summary>
///<returns>g[N</returns>
///-----------------------------------------------------------------------
char* Tokenizer::GetAsChar()
{
	return mToken;
}

///-----------------------------------------------------------------------
///<summary>
///g[Ndouble^Ƃĕԋp
///</summary>
///<returns>double^ɕϊg[N</returns>
///-----------------------------------------------------------------------
double Tokenizer::GetAsDouble()
{
	return atof( mToken );
}

///-----------------------------------------------------------------------
///<summary>
///g[Nfloat^Ƃĕԋp
///</summary>
///<returns>float^ɕϊg[N</returns>
///-----------------------------------------------------------------------
float Tokenizer::GetAsFloat()
{
	return static_cast<float>( atof( mToken) );
}

///-----------------------------------------------------------------------
///<summary>
///g[Nint^Ƃĕԋp
///</summary>
///<returns>int^ɕϊg[N</returns>
///-----------------------------------------------------------------------
int32_t Tokenizer::GetAsInt()
{
	return atoi( mToken );
}

///-----------------------------------------------------------------------
///<summary>
///̃g[N擾Cchar^Ƃĕԋp
///</summary>
///<returns>g[N</returns>
///-----------------------------------------------------------------------
char* Tokenizer::GetNextAsChar()
{
	Next();
	return GetAsChar();
}

///-----------------------------------------------------------------------
///<summary>
///̃g[N擾Cdouble^Ƃĕԋp.
///</summary>
///<returns>double^ɕϊg[N</returns>
///-----------------------------------------------------------------------
double Tokenizer::GetNextAsDouble()
{
	Next();
	return GetAsDouble();
}

///-----------------------------------------------------------------------
///<summary>
///̃g[N擾Cfloat^Ƃĕԋp.
///</summary>
///<returns>float^ɕϊg[N</returns>
///-----------------------------------------------------------------------
float Tokenizer::GetNextAsFloat()
{
	Next();
	return GetAsFloat();
}

///-----------------------------------------------------------------------
///<summary>
///̃g[N擾Cint^Ƃĕԋp.
///</summary>
///<returns>int^ɕϊg[N</returns>
///-----------------------------------------------------------------------
int32_t Tokenizer::GetNextAsInt()
{
	Next();
	return GetAsInt();
}

} // namespace Util
} // namespace Asura

// End Of File.
